package com.zhiLian.vehicle.controller;

import java.util.ArrayList;
import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.zhiLian.common.core.domain.Result;
import com.zhiLian.common.core.utils.poi.ExcelUtil;
import com.zhiLian.common.core.web.controller.BaseController;
import com.zhiLian.common.core.web.page.TableDataInfo;
import com.zhiLian.common.log.annotation.Log;
import com.zhiLian.common.log.enums.BusinessType;
import com.zhiLian.common.security.annotation.RequiresPermissions;
import com.zhiLian.vehicle.domain.FaultRecord;
import com.zhiLian.vehicle.domain.req.FaultRecordEditReq;
import com.zhiLian.vehicle.domain.req.FaultRecordQueryReq;
import com.zhiLian.vehicle.domain.req.FaultRecordReqVo;
import com.zhiLian.vehicle.domain.req.FaultRecordSaveReq;
import com.zhiLian.vehicle.domain.vo.FaultRecordVo;
import com.zhiLian.vehicle.service.FaultRecordService;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;


/**
 * 故障记录Controller
 *
 * @author chx
 * @date 2024-06-20
 */
@Api(tags = "故障记录")
@RestController
@RequestMapping("/faultRecord")
public class FaultRecordController extends BaseController {
    @Autowired
    private FaultRecordService faultRecordService;

    /**
     * 查询故障记录列表
     */
    @ApiOperation("获取故障记录列表")
//    @RequiresPermissions("client:faultRecord:list")
    @GetMapping("/list")
    public Result<TableDataInfo<FaultRecord>> list(FaultRecordQueryReq faultRecordQueryReq) {
        startPage();
        List<FaultRecord> list = faultRecordService.list(FaultRecord.queryBuild(faultRecordQueryReq));
        return getDataTable(list);
    }

    /**
     * 导出故障记录列表
     */
    @ApiOperation("导出故障记录列表")
//    @RequiresPermissions("client:faultRecord:export")
    @Log(title = "故障记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, FaultRecord faultRecord) {
        List<FaultRecord> list = faultRecordService.list(faultRecord);
        ExcelUtil<FaultRecord> util = new ExcelUtil<FaultRecord>(FaultRecord.class);
        util.exportExcel(response, list, "故障记录数据");
    }

    /**
     * 获取故障记录详细信息
     */
    @ApiOperation("获取故障记录详细信息")
//    @RequiresPermissions("client:faultRecord:query")
    @GetMapping(value = "/{id}")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class)
    public Result<FaultRecord> getInfo(@PathVariable("id") Long id) {
        return Result.success(faultRecordService.getById(id));
    }

    /**
     * 新增故障记录
     */
//    @RequiresPermissions("client:faultRecord:add")
    @Log(title = "故障记录", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation("新增故障记录")
    public Result<String> add(@RequestBody FaultRecordSaveReq faultRecordSaveReq) {
        return toAjax(faultRecordService.save(FaultRecord.saveBuild(faultRecordSaveReq)));
    }

    /**
     * 修改故障记录
     */
//    @RequiresPermissions("client:faultRecord:edit")
    @Log(title = "故障记录", businessType = BusinessType.UPDATE)
    @PutMapping("/{id}")
    @ApiOperation("修改故障记录")
    public Result<String> edit(@PathVariable Long id, @RequestBody FaultRecordEditReq faultRecordEditReq) {
        return toAjax(faultRecordService.updateById(FaultRecord.editBuild(id,faultRecordEditReq)));
    }

    /**
     * 删除故障记录
     */
//    @RequiresPermissions("client:faultRecord:remove")
    @Log(title = "故障记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    @ApiOperation("删除故障记录")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = String.class, example = "1,2,3,4")
    public Result<String> remove(@PathVariable List<Long> ids) {
        return toAjax(faultRecordService.removeBatchByIds(ids));
    }

    @Log(title = "图",businessType = BusinessType.DELETE)
    @PostMapping("/countList")
    @ApiOperation("图展示")
    public Result<List<FaultRecordVo>> countList(@RequestBody FaultRecordReqVo recordReqVo) {
        return Result.success(faultRecordService.countList(recordReqVo));
    }
}
