package com.zhiLian.vehicle.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;

import com.zhiLian.common.core.domain.Result;
import com.zhiLian.common.core.utils.poi.ExcelUtil;
import com.zhiLian.common.core.web.controller.BaseController;
import com.zhiLian.common.core.web.page.TableDataInfo;
import com.zhiLian.common.log.annotation.Log;
import com.zhiLian.common.log.enums.BusinessType;
import com.zhiLian.common.security.annotation.RequiresPermissions;
import com.zhiLian.vehicle.domain.MessageDetail;
import com.zhiLian.vehicle.domain.VehicleMessage;
import com.zhiLian.vehicle.domain.req.MessageDetailEditReq;
import com.zhiLian.vehicle.domain.req.MessageDetailQueryReq;
import com.zhiLian.vehicle.domain.req.MessageDetailSaveReq;
import com.zhiLian.vehicle.domain.vo.VehicleMessageVo;
import com.zhiLian.vehicle.service.MessageDetailService;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 报文详情Controller
 *
 * @author chx
 * @date 2024-06-26
 */
@Api(tags = "报文详情")
@RestController
@RequestMapping("/messageDetail")
public class MessageDetailController extends BaseController {
    @Autowired
    private MessageDetailService messageDetailService;

    /**
     * 查询报文详情列表
     */
    @ApiOperation("获取报文详情列表")
    @GetMapping("/list")
    public Result<TableDataInfo<MessageDetail>> list(MessageDetailQueryReq messageDetailQueryReq) {
        startPage();
        List<MessageDetail> list = messageDetailService.list(MessageDetail.queryBuild(messageDetailQueryReq));
        return getDataTable(list);
    }

    @PostMapping("addMessageIds")
    public Result addMessageIds(@RequestBody VehicleMessage vehicleMessageVo){
        return Result.success(messageDetailService.addMessageIds(vehicleMessageVo));
    }

    /**
     * 导出报文详情列表
     */
    @ApiOperation("导出报文详情列表")
    @Log(title = "报文详情", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, MessageDetail messageDetail) {
        List<MessageDetail> list = messageDetailService.list(messageDetail);
        ExcelUtil<MessageDetail> util = new ExcelUtil<MessageDetail>(MessageDetail.class);
        util.exportExcel(response, list, "报文详情数据");
    }



    /**
     * 获取报文详情详细信息
     */
    @ApiOperation("获取报文详情详细信息")
//    @RequiresPermissions("car:messageDetail:query")
    @GetMapping(value = "/{id}")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = Long.class)
    public Result<MessageDetail> getInfo(@PathVariable("id") Long id) {
        return Result.success(messageDetailService.getById(id));
    }

    /**
     * 新增报文详情
     */
//    @RequiresPermissions("car:messageDetail:add")
    @Log(title = "报文详情", businessType = BusinessType.INSERT)
    @PostMapping
    @ApiOperation("新增报文详情")
    public Result<String> add(@RequestBody MessageDetailSaveReq messageDetailSaveReq) {
        return toAjax(messageDetailService.save(MessageDetail.saveBuild(messageDetailSaveReq)));
    }

    /**
     * 修改报文详情
     */
//    @RequiresPermissions("car:messageDetail:edit")
    @Log(title = "报文详情", businessType = BusinessType.UPDATE)
    @PutMapping("/{id}")
    @ApiOperation("修改报文详情")
    public Result<String> edit(@PathVariable Long id, @RequestBody MessageDetailEditReq messageDetailEditReq) {
        return toAjax(messageDetailService.updateById(MessageDetail.editBuild(id,messageDetailEditReq)));
    }

    /**
     * 删除报文详情
     */
//    @RequiresPermissions("car:messageDetail:remove")
    @Log(title = "报文详情", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    @ApiOperation("删除报文详情")
    @ApiImplicitParam(name = "id", value = "id", required = true, dataType = "Long", paramType = "path", dataTypeClass = String.class, example = "1,2,3,4")
    public Result<String> remove(@PathVariable List<Long> ids) {
        return toAjax(messageDetailService.removeBatchByIds(ids));
    }
}
