package com.zhiLian.vehicle.domain;

import java.util.Date;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.zhiLian.common.core.annotation.Excel;
import com.zhiLian.vehicle.domain.req.FaultRecordEditReq;
import com.zhiLian.vehicle.domain.req.FaultRecordQueryReq;
import com.zhiLian.vehicle.domain.req.FaultRecordSaveReq;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.AllArgsConstructor;
import lombok.experimental.SuperBuilder;
import io.swagger.annotations.*;

/**
 * 故障记录对象 fault_record
 * @date 2024-06-20
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@TableName("fault_record")
@ApiModel(value = "FaultRecord", description = "故障记录")
public class FaultRecord {

    private static final long serialVersionUID = 1L;

    /** 主键 */
    @TableId(value = "id",type = IdType.AUTO)
    @ApiModelProperty(name = "主键", value = "主键")
    private Long id;

    /** 故障码 */
    @Excel(name = "故障码")
    @ApiModelProperty(name = "故障码", value = "故障码")
    private String faultCode;

    /** 车辆vin */
    @Excel(name = "车辆vin")
    @ApiModelProperty(name = "车辆vin", value = "车辆vin")
    private String vin;

    /** 故障开始时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "故障开始时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty(name = "故障开始时间", value = "故障开始时间")
    private Date startTime;

    /** 故障结束时间 */
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "故障结束时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty(name = "故障结束时间", value = "故障结束时间")
    private Date endTime;

    /** 故障级别 */
    @Excel(name = "故障级别")
    @ApiModelProperty(name = "故障级别", value = "故障级别")
    private String faultLevel;

    /** 故障是否处理 */
    @Excel(name = "故障是否处理")
    @ApiModelProperty(name = "故障是否处理", value = "故障是否处理")
    private String faultHandle;

    /**
    * 查询构造器
    */
    public static FaultRecord queryBuild( FaultRecordQueryReq faultRecordQueryReq){
        return FaultRecord.builder()
                .faultCode(faultRecordQueryReq.getFaultCode())
                .vin(faultRecordQueryReq.getVin())
                .build();
    }

    /**
    * 添加构造器
    */
    public static FaultRecord saveBuild(FaultRecordSaveReq faultRecordSaveReq){
        return FaultRecord.builder()
                .faultCode(faultRecordSaveReq.getFaultCode())
                .vin(faultRecordSaveReq.getVin())
                .startTime(faultRecordSaveReq.getStartTime())
                .endTime(faultRecordSaveReq.getEndTime())
                .faultLevel(faultRecordSaveReq.getFaultLevel())
                .faultHandle(faultRecordSaveReq.getFaultHandle())
                .build();
    }

    /**
    * 修改构造器
    */
    public static FaultRecord editBuild(Long id, FaultRecordEditReq faultRecordEditReq){
        return FaultRecord.builder()
            .id(id)
                .faultCode(faultRecordEditReq.getFaultCode())
                .vin(faultRecordEditReq.getVin())
                .startTime(faultRecordEditReq.getStartTime())
                .endTime(faultRecordEditReq.getEndTime())
                .faultLevel(faultRecordEditReq.getFaultLevel())
                .faultHandle(faultRecordEditReq.getFaultHandle())
                .build();
    }

}
