package com.zhiLian.vehicle.job;


import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.zhiLian.common.business.domain.Entinfo;
import com.zhiLian.common.business.remote.RemoteBusinessService;
import com.zhiLian.common.redis.service.RedisService;
import com.zhiLian.vehicle.datasource.config.holder.DynamicDataSourceHolder;
import com.zhiLian.vehicle.domain.MessageDetail;
import com.zhiLian.vehicle.domain.Vehicle;
import com.zhiLian.vehicle.domain.VehicleMessage;
import com.zhiLian.vehicle.mapper.MessageDetailMapper;
import com.zhiLian.vehicle.mapper.VehicleMapper;
import com.zhiLian.vehicle.service.IVehicleService;
import com.zhiLian.vehicle.service.MessageDetailService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.concurrent.Executor;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

/**
 * BingRui.Hou
 *
 * @Description 描述
 * @ClassName ManyJob
 * @Date 2024/06/20 17:34
 */
@Component
@Log4j2
public class ManyJob {

    @Autowired
    private RedisTemplate<String,String> redisTemplate;

    @Autowired
    private VehicleMapper vehicleService;

    @Autowired
    private RemoteBusinessService remoteBusinessService;

    @Autowired
    private MessageDetailMapper messageDetailMapper;

    @Autowired
    private MessageDetailService messageDetailService;

    @Scheduled(cron = "0 0 0 * * ?")// 每10分钟执行一次
    public void manyJob(){
        DynamicDataSourceHolder.setDynamicDataSourceKey("test_00");
        List<Vehicle> list = vehicleService.selectVehicleListAll(new Vehicle());
        ExecutorService executor = Executors.newFixedThreadPool(list.size());
        list.forEach(vehicle -> {
            executor.submit(new Runnable() {
                @Override
                public void run() {
                    redisTemplate
                            .opsForValue()
                            .set(vehicle.getNumber()+"1",
                            String.valueOf(vehicle.getBusinessId()));
                }
            });
        });
        DynamicDataSourceHolder.removeDynamicDataSourceKey();

    }


    @Scheduled(cron = "* */20 * * * ?")// 每10分钟执行一次
    public void manyJobMess(){
        List<Entinfo> list = remoteBusinessService.listAll();
        ExecutorService executorService = Executors.newFixedThreadPool(list.size());
        list.forEach(entinfo -> {
            executorService.submit(new Runnable() {
                @Override
                public void run() {
                    DynamicDataSourceHolder.setDynamicDataSourceKey(entinfo.getEntCode());
                    List<VehicleMessage> list1= messageDetailMapper.listVehicleMess();
                    list1.forEach(vehicleMessage -> {
                        redisTemplate.delete(vehicleMessage.getVin()+"List");
                        List<MessageDetail> messageDetailList = messageDetailService.list(new QueryWrapper<MessageDetail>().
                                in("id", Arrays.asList(vehicleMessage.getMessageIds().split(","))));
                        messageDetailList.forEach(messageDetail -> {
                            redisTemplate.opsForList().rightPush(vehicleMessage.getVin()+"List", JSON.toJSONString(messageDetail));
                        });
                    });
                    DynamicDataSourceHolder.removeDynamicDataSourceKey();
                }
            });
        });
    }

    public void redis(Entinfo entinfo){
        DynamicDataSourceHolder.setDynamicDataSourceKey(entinfo.getEntCode());
        List<VehicleMessage> list= messageDetailMapper.listVehicleMess();
        list.forEach(vehicleMessage -> {
            redisTemplate.delete(vehicleMessage.getVin()+"List");
            String[] split = vehicleMessage.getMessageIds().split(",");
            ArrayList<MessageDetail> messageDetailArrayList = new ArrayList<>();
            for (String s : split) {
                MessageDetail one = messageDetailMapper.selectOne(
                        new LambdaQueryWrapper<MessageDetail>()
                                .eq(MessageDetail::getId, Long.valueOf(s)));
                messageDetailArrayList.add(one);
            }
            messageDetailArrayList.forEach(messageDetail -> {
                redisTemplate.opsForList().rightPush(vehicleMessage.getVin()+"List", JSON.toJSONString(messageDetail));
            });
        });
        DynamicDataSourceHolder.removeDynamicDataSourceKey();
    }

}
