package com.zhiLian.vehicle.rabbitmq.producer;


import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.rabbitmq.client.Channel;
import com.zhiLian.common.business.domain.Entinfo;
import com.zhiLian.common.business.remote.RemoteBusinessService;
import com.zhiLian.vehicle.datasource.config.holder.DynamicDataSourceHolder;
import com.zhiLian.vehicle.domain.FaultRecord;
import com.zhiLian.vehicle.domain.MessageDetail;
import com.zhiLian.vehicle.domain.Vehicle;
import com.zhiLian.vehicle.domain.VehicleMessage;
import com.zhiLian.vehicle.mapper.GroupMapper;
import com.zhiLian.vehicle.mapper.MessageDetailMapper;
import com.zhiLian.vehicle.mapper.VehicleMapper;
import com.zhiLian.vehicle.service.FaultRecordService;
import com.zhiLian.vehicle.service.MessageDetailService;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.Queue;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @BelongsProject: Bob_Up_Like_A_Cork
 * @BelongsPackage: com.bwie.sms.config
 * @Author: zhangquan
 * @CreateTime: 2023/7/30 20:48
 */
@Component
@Log4j2
public class VehicleProducer {
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private  FaultRecordService faultRecordService;

    @Autowired
    private VehicleMapper vehicleService;


    @Autowired
    private RemoteBusinessService remoteBusinessService;

    @Autowired
    private MessageDetailMapper messageDetailMapper;
    @Autowired
    private final static String ZHILIAN_END="zhiLian-vehicle-end";
    private final static String ZHILIAN_VIN="zhiLian-vehicle-vin";
    private final static String ZHILIAN_START="zhiLian-vehicle-start";



    //调用注解  添加队列名称
    @RabbitListener(queuesToDeclare = {@Queue(name = ZHILIAN_START)})
    public void smsConfigStart(String msg, Message message, Channel channel){
        //获取消息的ID
        String messageId = message.getMessageProperties().getMessageId();
            try {
                Long count = redisTemplate.opsForSet().add("messageId", messageId);
                if (count==1) {
                    log.info("故障开始消费！{}", msg);
                    manyJob();
                    FaultRecord faultRecord = JSON.parseObject(msg, FaultRecord.class);
                    //判断车辆属于哪个企业
                    String s = redisTemplate.opsForValue().get(faultRecord.getVin()+"1");
                    // 进行添加故障表
                    Executors.newFixedThreadPool(1).execute(new Runnable() {
                        @Override
                        public void run() {
                            //选择数据源，切换数据源，
                            DynamicDataSourceHolder.setDynamicDataSourceKey("test_"+Integer.valueOf(s));
                            FaultRecord build = FaultRecord
                                    .builder()
                                    .faultCode(faultRecord.getFaultCode())
                                    .vin(faultRecord.getVin())
                                    .startTime(faultRecord.getStartTime()).build();
                            FaultRecord faultRecordOne = faultRecordService.getByFaultRecord(faultRecord);
                            log.info("查询到的故障为：{}",faultRecordOne);
                            if (faultRecordOne==null){
                                faultRecordService.save(build);
                            }
                            // 移除数据源，
                            DynamicDataSourceHolder.removeDynamicDataSourceKey();
                        }
                    });
                    channel.basicAck(message.getMessageProperties().getDeliveryTag(), false);
                    log.info("消费成功！数据源为：{}",message);
                }
            } catch (IOException e) {
                log.info("消费失败，{}",e.getMessage());
                try {
                    //回退消息
                    channel.basicReject(message.getMessageProperties().getDeliveryTag(),false);
                    log.info("回退成功");
                }catch (IOException ex){
                    log.info("回退失败:{}",ex.getMessage());
                }
                throw new RuntimeException(e);
            }
        }


    @RabbitListener(queuesToDeclare = {@Queue(name = ZHILIAN_END)})
    public void smsConfigEnt(String msg, Message message, Channel channel){
        //获取消息的ID
        String messageId = message.getMessageProperties().getMessageId();
        try {
            //添加消息id到redis set集合中  添加成功返回1 表示未消费    添加失败返回0 表示已消费
            Long count = redisTemplate.opsForSet().add("messageId", messageId);
            //添加成功  正常消费信息
            if (count == 1) {
                log.info("故障结束消费:{}",msg);
                manyJob();
                FaultRecord faultRecord = JSON.parseObject(msg, FaultRecord.class);
                //判断车辆属于哪个企业
                String s = redisTemplate.opsForValue().get(faultRecord.getVin()+"1");

                Executors.newFixedThreadPool(1).execute(new Runnable() {
                    @Override
                    public void run() {
                        //选择数据源，切换数据源，
                        DynamicDataSourceHolder.setDynamicDataSourceKey("test_"+Integer.valueOf(s));
                        // 进行查询故障表
                        FaultRecord faultRecordOne = faultRecordService.getByFaultRecord(faultRecord);
                        log.info("查询到的故障为：{}",faultRecordOne);
                        faultRecordOne.setEndTime(faultRecord.getEndTime());
                        // 进行修改故障表
                        faultRecordService.updateByFaultRecord(faultRecordOne);
                        // 移除数据源，
                        DynamicDataSourceHolder.removeDynamicDataSourceKey();
                    }
                });
                //确认消费
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
                log.info("消费成功");
            }
        } catch (Exception e) {
            //删除队列ID
            log.info("消费失败，{}",e.getMessage());
            try {
                //回退消息
                channel.basicReject(message.getMessageProperties().getDeliveryTag(),false);
                log.info("回退消息");
            } catch (IOException ex) {
                //回退失败
                log.info("回退失败");
            }
        }
    }

    @Autowired
    private MessageDetailService messageDetailService;

    @RabbitListener(queuesToDeclare = {@Queue(name = ZHILIAN_VIN)})
    public void smsConfigVin(String msg, Message message, Channel channel){
        //获取消息的ID
        String messageId = message.getMessageProperties().getMessageId();
        try {
            //添加消息id到redis set集合中  添加成功返回1 表示未消费    添加失败返回0 表示已消费
            Long count = redisTemplate.opsForSet().add("messageId", messageId);
            //添加成功  正常消费信息
            if (count == 1) {
                log.info("消费:{}",msg);
                manyJob();
//                //判断车辆属于哪个企业
                String s = redisTemplate.opsForValue().get(msg+"1");
                redisTemplate.delete(msg+"List");
                Executors.newFixedThreadPool(1).execute(new Runnable() {
                    @Override
                    public void run() {
                        DynamicDataSourceHolder.setDynamicDataSourceKey("test_"+s);
                        VehicleMessage vehicleMessage = messageDetailMapper.findByVin(VehicleMessage.builder().vin(msg).build());
                        List<MessageDetail> messageDetailList = messageDetailService.list(new QueryWrapper<MessageDetail>().
                                in("id", Arrays.asList(vehicleMessage.getMessageIds().split(","))));
                        messageDetailList.forEach(messageDetail -> {
                            redisTemplate.opsForList().rightPush(msg+"List",JSON.toJSONString(messageDetail));
                        });
                        DynamicDataSourceHolder.removeDynamicDataSourceKey();
                    }
                });
                //确认消费
                channel.basicAck(message.getMessageProperties().getDeliveryTag(),false);
                log.info("消费成功");
            }
        } catch (Exception e) {
            //删除队列ID
            log.info("消费失败，{}",e.getMessage());
            try {
                //回退消息
                channel.basicReject(message.getMessageProperties().getDeliveryTag(),false);
                log.info("回退消息");
            } catch (IOException ex) {
                //回退失败
                log.info("回退失败");
            }
        }
    }



    public void manyJobMess(){
        List<Entinfo> list = remoteBusinessService.listAll();
        ExecutorService executorService = Executors.newFixedThreadPool(list.size());
        list.forEach(entinfo -> {
            executorService.submit(new Runnable() {
                @Override
                public void run() {
                    DynamicDataSourceHolder.setDynamicDataSourceKey(entinfo.getEntCode());
                    List<VehicleMessage> list= messageDetailMapper.listVehicleMess();
                    list.forEach(vehicleMessage -> {
                        redisTemplate.delete(vehicleMessage.getVin()+"List");
                        String messageIds = vehicleMessage.getMessageIds();
                        List<MessageDetail> messageDetails = messageDetailMapper.selectList(new LambdaQueryWrapper<MessageDetail>().in(MessageDetail::getId, messageIds));
                        messageDetails.forEach(messageDetail -> {
                            redisTemplate.opsForList().rightPush(vehicleMessage.getVin()+"List", JSON.toJSONString(messageDetail));
                        });
                    });
                    DynamicDataSourceHolder.removeDynamicDataSourceKey();
                }
            });
        });
    }

    public void manyJob(){
        DynamicDataSourceHolder.setDynamicDataSourceKey("test_00");
        List<Vehicle> list = vehicleService.selectVehicleListAll(new Vehicle());
        ExecutorService executor = Executors.newFixedThreadPool(list.size());
        list.forEach(vehicle -> {
            executor.submit(new Runnable() {
                @Override
                public void run() {
                    redisTemplate
                            .opsForValue()
                            .set(vehicle.getNumber()+"1",
                                    String.valueOf(vehicle.getBusinessId()));
                }
            });
        });
        DynamicDataSourceHolder.removeDynamicDataSourceKey();
    }

}
