package com.zhiLian.vehicle.service.impl;


import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhiLian.common.core.domain.Result;
import com.zhiLian.common.redis.service.RedisService;
import com.zhiLian.vehicle.datasource.config.holder.DynamicDataSourceHolder;
import com.zhiLian.vehicle.domain.MessageDetail;
import com.zhiLian.vehicle.domain.VehicleMessage;
import com.zhiLian.vehicle.domain.vo.VehicleMessageVo;
import com.zhiLian.vehicle.mapper.MessageDetailMapper;
import com.zhiLian.vehicle.service.MessageDetailService;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.weaver.ast.Or;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.concurrent.Executor;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * 报文详情Service业务层处理
 *
 * @author chx
 * @date 2024-06-26
 */
@Slf4j
@Service
public class MessageDetailServiceImpl extends ServiceImpl<MessageDetailMapper, MessageDetail> implements MessageDetailService {

    @Autowired
    private  MessageDetailMapper messageDetailMapper;

    /**
     * redis 缓存
     */
    @Autowired
    private RedisTemplate<String,String> redisTemplate;
    /**
     * 查询报文详情列表
     *
     * @param messageDetail 报文详情
     * @return 报文详情
     */
    @Override
    public List<MessageDetail> list(MessageDetail messageDetail) {
        LambdaQueryWrapper<MessageDetail> queryWrapper = new LambdaQueryWrapper<>();
        return list(queryWrapper);
    }

    @Autowired
    private RedisService redisService;
    @Override
    public Result addMessageIds(VehicleMessage vehicleMessageVo) {
        redisTemplate.delete(vehicleMessageVo.getVin()+"List");
        VehicleMessage vehicleMessage=messageDetailMapper.findByVin(vehicleMessageVo);
        redis(vehicleMessageVo);
        rabbitTemplate.convertAndSend ( "zhiLian-vehicle-return",vehicleMessage.getVin() , messag ->{
            messag.getMessageProperties ( ).setMessageId ( UUID.randomUUID ( ).toString ( ) );
            log.info ( "消息发送成功" );
            return messag;
        });
        //判断  新增/ 修改
        if (vehicleMessage!=null){
            //修改
            Boolean b=messageDetailMapper.updateByVin(vehicleMessageVo);
            return Result.success();
        }
        //新增
        Boolean b = messageDetailMapper.addMess(vehicleMessageVo);
        return null;
    }

    @Autowired
    private MessageDetailService messageDetailService;

    @Autowired
    private RabbitTemplate rabbitTemplate;

    /**
     * 报文配置存入redis
     * @param vehicleMessageVo
     */
    private void redis(VehicleMessage vehicleMessageVo) {
        List<MessageDetail> messageDetailList = messageDetailService.list(new QueryWrapper<MessageDetail>().
                   in("id", Arrays.asList(vehicleMessageVo.getMessageIds().split(","))));
        messageDetailList.forEach(messageDetail -> {
                   redisTemplate.opsForList().rightPush(vehicleMessageVo.getVin()+"List", JSON.toJSONString(messageDetail));
        });
    }
}
